/*
 * B E A   S Y S T E M S
 *
 * Copyright (c) 2003-2004  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to
 * disclose its contents, reproduce it, or use,  or license the use, for
 * manufacture or sale, the information or anything described therein. Any
 * use, disclosure, or reproduction without BEA System's prior written
 * permission is strictly prohibited.
 *
 */

package com.bea.netuix.util;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import com.vladium.utils.IObjectProfileNode;
import com.vladium.utils.ObjectProfiler;

import com.bea.netuix.nf.UIControl;
import com.bea.netuix.nf.container.jsp.tags.JspUIControlFactory;
import com.bea.netuix.nf.container.jsp.tags.MetaUIControl;
import com.bea.netuix.nf.container.jsp.tags.ReflectionMetaControlFactory;
import com.bea.netuix.servlets.manager.SingleFileProcessor;

public class PortalInfoBean
{
    private String portalFilename;

    private String completeURL;

    private long metaUIControlTreeSize;

    private long uiControlTreeSize;

    private Map uiControlTreeStatistics;

    private long responseSize;

    private StringBuffer errorMessage = new StringBuffer();

    public String getPortalFilename()
    {
        return portalFilename;
    }

    public void setPortalFilename(String portalFilename)
    {
        this.portalFilename = portalFilename;
    }

    public String getCompleteURL()
    {
        return completeURL;
    }

    public void setCompleteURL(String completeURL)
    {
        this.completeURL = completeURL;
    }

    public long getMetaUIControlTreeSize()
    {
        return metaUIControlTreeSize;
    }

    public long getUiControlTreeSize()
    {
        return uiControlTreeSize;
    }

    public Map getUiControlTreeStatistics()
    {
        return uiControlTreeStatistics;
    }

    public long getResponseSize()
    {
        return responseSize;
    }

    public String getErrorMessage()
    {
        return errorMessage.toString();
    }

    public boolean validate()
    {
        boolean result = true;

        if (getPortalFilename().endsWith(".portal") == false)
        {
            errorMessage.append("Portal File should end with .portal extension \n");
            result = false;
        }

        return result;
    }

    public void computeStatistics() throws IOException
    {
        responseSize = computeResponseSize(getCompleteURL());

        computePortalObjectModelStatistics();
    }

    private long computeResponseSize(String url) throws IOException
    {
        URL portalURL = new URL(url);

        URLConnection connection = portalURL.openConnection();

        long counter = 0;

        if (connection instanceof HttpURLConnection)
        {
            HttpURLConnection httpURLConnection = (HttpURLConnection)connection;

            httpURLConnection.connect();

            InputStream inputStream = null;

            try
            {
                inputStream = httpURLConnection.getInputStream();

                while (inputStream.read() != -1)
                {
                    counter = counter + 1;
                }
            }
            finally
            {
                if (inputStream != null)
                {
                    try
                    {
                        inputStream.close();
                    }
                    catch (IOException e)
                    {
                        //ignore
                    }
                }

                httpURLConnection.disconnect();
            }
        }

        return counter;
    }

    private void computePortalObjectModelStatistics()
    {
        MetaUIControl root = getRootMetaUIControl(getPortalFilename());

        metaUIControlTreeSize = computeSize(root);

        UIControl uiControlRoot = root.createControlTree();

        uiControlTreeSize = computeSize(uiControlRoot);

        uiControlTreeStatistics = computeUIControlStatistics(uiControlRoot);
    }

    private long computeSize(Object root)
    {
        IObjectProfileNode iObjectProfileNode = ObjectProfiler.profile(root);

        return iObjectProfileNode.size();
    }

    private MetaUIControl getRootMetaUIControl(String filename)
    {
        JspUIControlFactory jspUIControlFactory = SingleFileProcessor.getMergedControlFactoryFromFile(filename);

        if (jspUIControlFactory != null && jspUIControlFactory instanceof ReflectionMetaControlFactory)
        {
            ReflectionMetaControlFactory metaControlFactory = (ReflectionMetaControlFactory)jspUIControlFactory;

            return metaControlFactory.getMetaRoot();
        }
        else
        {
            throw new IllegalStateException("Couldn't retrieve necessary object model for the given portal file <" + filename + ">");
        }
    }

    private Map computeUIControlStatistics(UIControl root)
    {
        // key --> className, value --> count
        // using linked hash map to maintain insertion order
        Map statistics = new LinkedHashMap();

        traverseUIControlTree(statistics, root);

        return statistics;
    }

    private void traverseUIControlTree(Map stats, UIControl control)
    {
        Class clazz = control.getClass();

        Integer counter = (Integer)stats.get(clazz);

        if (counter == null)
        {
            counter = new Integer(1);
        }
        else
        {
            counter = new Integer(counter.intValue() + 1);
        }

        stats.put(clazz, counter);

        // Handle Children
        Iterator iterator = control.getChildren();

        while (iterator.hasNext())
        {
            UIControl uiControl = (UIControl) iterator.next();

            traverseUIControlTree(stats, uiControl);
        }
    }
}
